#! /bin/sh

APPS_NAME="DSL API Driver"

if [ -e sysconfig.sh ]; then
	. sysconfig.sh
	. config.sh
	. model_config.sh
	. gen_funcs.sh
	. build_functions.sh 
else
        echo "Application "$APPS_NAME" not configured"
        exit 0
fi
echo "----------------------------------------------------------------------"
echo "-------------------- build $APPS_NAME ----------------------------"
echo "----------------------------------------------------------------------"

parse_args $@

cd drv_dsl_cpe_api

if [ $BUILD_CLEAN -eq 1 ]; then
	if [ -e .config_ok ]; then
		make distclean
		rm -f .config_ok
	fi
	[ ! $BUILD_CONFIGURE -eq 1 ] && exit 0
fi
if [ ! -f .config_ok -o $BUILD_CONFIGURE -eq 1 ]; then
	if [ ! -e configure ]; then
		autoreconf -i
		if [ ! -e NEWS ]; then
		    touch NEWS
		fi
		if [ ! -e README ]; then
		    touch README
		fi
		if [ ! -e AUTHORS ]; then
		    touch AUTHORS
		fi
		if [ ! -e ChangeLog ]; then
		    touch ChangeLog
		fi
		if [ ! -e TODO ]; then
		    touch TODO
		fi
	fi
fi

ADD_DRV_CFLAGS="-DMODVERSIONS -Os -G 0 -mno-abicalls -mlong-calls -fno-pic \
 -finline-limit=100000 -mabi=32 -march=mips32 -Wa,-32 -Wa,-march=mips32 \
 -Wa,-mips32 -Wa,--trap"

# call build script with defined settings
../build_driver.sh \
    --arch=mips \
    --add-drv-cflags="$ADD_DRV_CFLAGS" \
    --device=amazon-se \
    --channels=1 \
    --inc-debug=1 \
    --inc-debug-prints=0 \
    --inc-adsl-mib=1 \
    --inc-adsl-trace=0 \
    --inc-adsl-led=1 \
    --adsl-data-led-freq=4 \
    --inc-dsl-ceoc=0 \
    --inc-dsl-static-delt=0 \
    --inc-dsl-pm=1 \
    --inc-dsl-pm-total-counters=1 \
    --inc-dsl-pm-showtime-counters=1 \
    --inc-dsl-pm-history=1 \
    --inc-dsl-pm-line-counters=1 \
    --inc-dsl-pm-channel-counters=1 \
    --inc-dsl-pm-datapath-counters=1 \
    --inc-dsl-pm-line-thresholds=0 \
    --inc-dsl-pm-channel-thresholds=0 \
    --inc-dsl-pm-datapath-thresholds=0 \
    --inc-dsl-pm-optional-parameters=0 \
    --inc-firmware-memory-free=1
cd -
