/******************************************************************************

                               Copyright (c) 2007
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

#define DSL_INTERN

#include "drv_dsl_cpe_api.h"

#ifdef INCLUDE_DSL_CPE_PROCFILES

#include "drv_dsl_cpe_procfs_linux.h"

#ifdef __cplusplus
   extern "C" {
#endif

#undef DSL_DBG_BLOCK
#define DSL_DBG_BLOCK DSL_DBG_OS

static DSL_int_t DSL_DRV_PROC_AvsarVerShow(struct seq_file* sfile);
static DSL_int_t DSL_DRV_PROC_AvsarModemTrainingShow(struct seq_file* sfile);
static DSL_int_t DSL_DRV_PROC_AvsarModemStatsShow(struct seq_file* sfile);
#ifdef INCLUDE_DSL_G997_PER_TONE
static DSL_void_t* DSL_DRV_PROC_BitAllocationTableDataGet(struct seq_file* sfile);
static DSL_int_t DSL_DRV_PROC_BitAllocationTableShow(struct seq_file* sfile);
static DSL_void_t* DSL_DRV_PROC_SnrDsAllocationTableDataGet(struct seq_file* sfile);
static DSL_int_t DSL_DRV_PROC_SnrDsAllocationTableShow(struct seq_file* sfile);
#endif /* INCLUDE_DSL_G997_PER_TONE*/
#ifdef INCLUDE_DSL_DELT
static DSL_void_t* DSL_DRV_PROC_QlnDsShowtimeDataGet(struct seq_file* sfile);
static DSL_int_t DSL_DRV_PROC_QlnDsShowtimeShow(struct seq_file* sfile);
#endif /* INCLUDE_DSL_DELT*/


DSL_DRV_PROC_EntryTable_t procEntryTable[] =
{
   /* "avsar_ver" proc entry, device dependant*/
   DSL_DRV_PROC_ENTRY_ADD("avsar_ver", DSL_DRV_PROC_AvsarVerShow, DSL_NULL, DSL_TRUE),
   /* "avsar_modem_training" proc entry, device dependant*/
   DSL_DRV_PROC_ENTRY_ADD("avsar_modem_training", DSL_DRV_PROC_AvsarModemTrainingShow, DSL_NULL, DSL_TRUE),
   /* "avsar_modem_stats" proc entry, device dependant*/
   DSL_DRV_PROC_ENTRY_ADD("avsar_modem_stats", DSL_DRV_PROC_AvsarModemStatsShow, DSL_NULL, DSL_TRUE),
#ifdef INCLUDE_DSL_G997_PER_TONE
   /* "avsar_bit_allocation_table" proc entry, device dependant*/
   DSL_DRV_PROC_ENTRY_ADD("avsar_bit_allocation_table", DSL_DRV_PROC_BitAllocationTableShow, DSL_DRV_PROC_BitAllocationTableDataGet, DSL_TRUE),
   /* "avsar_SNRpsds" proc entry, device dependant*/
   DSL_DRV_PROC_ENTRY_ADD("avsar_SNRpsds", DSL_DRV_PROC_SnrDsAllocationTableShow, DSL_DRV_PROC_SnrDsAllocationTableDataGet, DSL_TRUE),
#endif /* INCLUDE_DSL_G997_PER_TONE*/
#ifdef INCLUDE_DSL_DELT
   /* "avsar_QLNpsds" proc entry, device dependant*/
   DSL_DRV_PROC_ENTRY_ADD("avsar_QLNpsds", DSL_DRV_PROC_QlnDsShowtimeShow, DSL_DRV_PROC_QlnDsShowtimeDataGet, DSL_TRUE),
#endif /* INCLUDE_DSL_DELT*/
   /* delimeter*/
   DSL_DRV_PROC_ENTRY_ADD(DSL_NULL, DSL_NULL, DSL_NULL, DSL_FALSE)
};

static void *DSL_DRV_SeqStart(struct seq_file *sfile, loff_t *pPos);
static void *DSL_DRV_SeqNext(struct seq_file *sfile, void *v, loff_t *pPos);
static void DSL_DRV_SeqStop(struct seq_file *sfile, void *v);
static int DSL_DRV_SeqShow(struct seq_file *sfile, void *v);
static int DSL_DRV_SeqOpen(struct inode *inode, struct file *pFile);

#ifndef _lint
static struct seq_operations DSL_DRV_SeqOps = 
{
   .start = DSL_DRV_SeqStart,
   .next  = DSL_DRV_SeqNext,
   .stop  = DSL_DRV_SeqStop,
   .show  = DSL_DRV_SeqShow
};

static struct file_operations DSL_DRV_ProcOps = 
{
   .owner   = THIS_MODULE,
   .open    = DSL_DRV_SeqOpen,
   .read    = seq_read,
   .llseek  = seq_lseek,
   .release = seq_release
};
#else
static struct seq_operations DSL_DRV_SeqOps = 
{
   /*.start = */DSL_DRV_SeqStart,
   /*.next  = */DSL_DRV_SeqNext,
   /*.stop  = */DSL_DRV_SeqStop,
   /*.show  = */DSL_DRV_SeqShow
};

static struct file_operations DSL_DRV_ProcOps = 
{
   /*.owner   = THIS_MODULE,*/
   /*.open    = */  DSL_DRV_SeqOpen,
   /*.release = */  seq_release,
   /*.read    = */  seq_read,
   /*.write    = */ DSL_NULL,
   /*.llseek  = */  seq_lseek,
   /*.ioctl  = */   DSL_NULL,
   /*.poll  = */    DSL_NULL
};
#endif /* _lint*/

static void *DSL_DRV_SeqStart(struct seq_file *sfile, loff_t *pPos)
{
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;

   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);
      return NULL;
   }


   if ((pDevEntry->pProcDataGet != DSL_NULL) && (pDevEntry->SecPrivateData.bEnd != DSL_TRUE))
   {
      if (pDevEntry->SecPrivateData.pPrivateData == DSL_NULL && 
          pDevEntry->SecPrivateData.pPrintData   == DSL_NULL)
      {
         /* Get entry data*/
         pDevEntry->SecPrivateData.pPrivateData = pDevEntry->pProcDataGet(sfile);
         pDevEntry->SecPrivateData.pPrintData   = pDevEntry->SecPrivateData.pPrivateData;
      }
   }

   /* Start -> Stop sequence ends here*/
   if (pDevEntry->SecPrivateData.bEnd)
   {
      pDevEntry->SecPrivateData.bEnd         = DSL_FALSE;
      pDevEntry->SecPrivateData.v            = 0;
      pDevEntry->SecPrivateData.pPrintData   = DSL_NULL;
      pDevEntry->SecPrivateData.pPrivateData = DSL_NULL;
      /* no more iterations at the current sequence*/
      return NULL;
   }

   return (void *)pPos;
}

static void *DSL_DRV_SeqNext(struct seq_file *sfile, void *v, loff_t *pPos)
{
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;

   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return NULL;
   }

   (*pPos)++;

   return pDevEntry->SecPrivateData.bEnd ? DSL_NULL : (void *)pPos;
}

static void DSL_DRV_SeqStop(struct seq_file *sfile, void *v)
{
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;

   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return;
   }

   if (pDevEntry->SecPrivateData.bEnd)
   {
      if (pDevEntry->SecPrivateData.pPrivateData)
      {
         /* Free Private data at the end*/
         kfree(pDevEntry->SecPrivateData.pPrivateData);
         pDevEntry->SecPrivateData.pPrivateData = DSL_NULL;
      }
   }
   
   return;
}

static int DSL_DRV_SeqShow(struct seq_file *sfile, void *v)
{
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;

   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);
      return 0;
   }

   (void)pDevEntry->pProcShow(sfile);

   return 0;
}

static int DSL_DRV_SeqOpen(struct inode *inode, struct file *pFile)
{
   int retVal;

   /* open seq-file */
   retVal = seq_open(pFile, &DSL_DRV_SeqOps);
   if (retVal < 0)
   {
      return retVal;
   }

#ifndef _lint
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
   /* get proc-file entry and add device number to seq_file struct for later processing */
   ((struct seq_file *)pFile->private_data)->private = ((struct proc_dir_entry *)inode->u.generic_ip)->data;
#else
   /* FIX: Kernel 2.6, still not tested */
   ((struct seq_file *)pFile->private_data)->private = PROC_I(inode)->pde->data;
#endif
#endif /*_lint*/

   return retVal;
}

DSL_int_t DSL_DRV_InstallProcEntry(DSL_void_t)
{
   DSL_int_t i;
   DSL_char_t path[255];
   struct proc_dir_entry *driver_proc_node_base;
   struct proc_dir_entry *driver_proc_node;
   struct proc_dir_entry *driver_dev_proc_node;
   DSL_DRV_PROC_EntryTable_t *pProcEntryTable = procEntryTable;

   /* install the proc entry */
   printk(DSL_DRV_CRLF"DSL_DRV: using proc fs"DSL_DRV_CRLF);

   if (pProcEntryTable == DSL_NULL)
   {
      printk("DSL_DRV: proc table not found!" DSL_DRV_CRLF);
      return -EFAULT;
   }

   driver_proc_node_base = (DSL_DRV_PROC_DirEntry_t*)proc_mkdir( "driver/"DRV_DSL_CPE_API_DEV_NAME, NULL);
   
   if (driver_proc_node_base == NULL)
   {
      printk("DSL_DRV: cannot create driver base proc node!" DSL_DRV_CRLF);
      return -EFAULT;
   }

   /* Process the whole proc entry table*/
   while(pProcEntryTable->pProcShow && pProcEntryTable->string)
   {
      if (pProcEntryTable->bDevSpecific)
      {
         sprintf(path, "driver/%s/%s", DRV_DSL_CPE_API_DEV_NAME, pProcEntryTable->string);
         driver_proc_node = (DSL_DRV_PROC_DirEntry_t*) proc_mkdir( path, NULL);

         if (driver_proc_node == NULL)
         {
            printk("DSL_DRV: cannot create driver proc node for %s entry!" DSL_DRV_CRLF,
               pProcEntryTable->string);
            return -EFAULT;
         }

         /* Make proc device nodes*/
         for (i = 0; i < DSL_DRV_MAX_DEVICE_NUMBER; i++)
         {
            sprintf(path, "%02d", i);
            pProcEntryTable->entryData[i].devNum = i;
            pProcEntryTable->entryData[i].pProcShow = pProcEntryTable->pProcShow;
            pProcEntryTable->entryData[i].pProcDataGet = pProcEntryTable->pProcDataGet;
            pProcEntryTable->entryData[i].SecPrivateData.bEnd = DSL_FALSE;
            pProcEntryTable->entryData[i].SecPrivateData.v = 0;
            pProcEntryTable->entryData[i].SecPrivateData.pPrintData = DSL_NULL;
            pProcEntryTable->entryData[i].SecPrivateData.pPrivateData = DSL_NULL;

            driver_dev_proc_node = create_proc_entry(
                                      path, (S_IFREG|S_IRUGO), driver_proc_node);
            driver_dev_proc_node->proc_fops = &DSL_DRV_ProcOps;
            driver_dev_proc_node->data      = (void *)&(pProcEntryTable->entryData[i]);
         }
      }
      else
      {
         sprintf(path, "driver/%s", DRV_DSL_CPE_API_DEV_NAME);
         driver_proc_node = (DSL_DRV_PROC_DirEntry_t*) proc_mkdir( path, NULL);

         if (driver_proc_node == NULL)
         {
            printk("DSL_DRV: cannot create driver proc node!" DSL_DRV_CRLF);
            return -EFAULT;
         }
         
         pProcEntryTable->entryData[0].devNum = 0;
         pProcEntryTable->entryData[0].pProcShow = pProcEntryTable->pProcShow;
         pProcEntryTable->entryData[0].pProcDataGet = pProcEntryTable->pProcDataGet;
         pProcEntryTable->entryData[0].SecPrivateData.bEnd = DSL_FALSE;
         pProcEntryTable->entryData[0].SecPrivateData.v = 0;
         pProcEntryTable->entryData[0].SecPrivateData.pPrintData = DSL_NULL;
         pProcEntryTable->entryData[0].SecPrivateData.pPrivateData = DSL_NULL;

         driver_dev_proc_node = create_proc_entry(
                                   pProcEntryTable->string, (S_IFREG|S_IRUGO), driver_proc_node);
         
         driver_dev_proc_node->proc_fops = &DSL_DRV_ProcOps;
         driver_dev_proc_node->data      = (void *)&(pProcEntryTable->entryData[0]);
      }

      /* Move to the next entry*/
      pProcEntryTable++;
   }

   return 0;
}

DSL_int_t DSL_DRV_RemoveProcEntry(DSL_void_t)
{
   DSL_int_t i;
   char path[255];
   DSL_DRV_PROC_EntryTable_t *pProcEntryTable = procEntryTable;
   
   if (pProcEntryTable == DSL_NULL)
   {
      printk("DSL_DRV: proc table not found!" DSL_DRV_CRLF);
   }
   else
   {
      /* Process the whole proc entry table*/
      while(pProcEntryTable->pProcShow && pProcEntryTable->string)
      {
         if (pProcEntryTable->bDevSpecific)
         {
            for (i = 0; i < DSL_DRV_MAX_DEVICE_NUMBER; i++)
            {
               sprintf(path, "driver/%s/%s/%02d", DRV_DSL_CPE_API_DEV_NAME, pProcEntryTable->string, i);
               remove_proc_entry(path ,0);
            }

            sprintf(path, "driver/%s/%s", DRV_DSL_CPE_API_DEV_NAME, pProcEntryTable->string);
            remove_proc_entry(path ,0);
         }
         else
         {
            sprintf(path, "driver/%s/%s", DRV_DSL_CPE_API_DEV_NAME, pProcEntryTable->string);
            remove_proc_entry(path ,0);
         }

         /* Move to the next entry*/
         pProcEntryTable++;
      }
   
      sprintf(path, "driver/%s", DRV_DSL_CPE_API_DEV_NAME);
      remove_proc_entry(path ,0);
   }
  
   return 0;
}

static DSL_char_t* DSL_DRV_Itoa(DSL_int_t val)
{
   const DSL_char_t digits[] = "0123456789";
   DSL_int_t _val = val, i;
   static DSL_char_t buf[12] = {0};
   DSL_boolean_t bNegative = DSL_FALSE;

   if (_val == 0)
   {
      return "0";
   }
	
   if (_val < 0)
   {
      _val = -val;
      bNegative = DSL_TRUE;
   }
	
   for (i = 11; _val && i ; --i) 
   {
      buf[i] = digits[_val % 10];
      _val /= 10;
   }

   buf[i] = "-"[0];
   
   return bNegative ? &buf[i] : &buf[i+1];
}

/* Use this to format arrays of unspecified contents as hex values */
static DSL_void_t DSL_DRV_ArraySPrintF(
   DSL_char_t *pDst,
   DSL_void_t *pSrc,
   DSL_uint16_t nSrcSize,
   DSL_uint16_t nSrcElementSize,
   DSL_DRV_ArrayPrintFormat_t nFormat)
{
   DSL_uint16_t i = 0, j = 0, elements = 0;
   DSL_uint32_t nVal = 0;
   DSL_char_t c;
   DSL_int_t ret = 0;

   elements = nSrcSize/nSrcElementSize;

   if (nFormat == DSL_ARRAY_FORMAT_HEX)
   {
      ret = sprintf(pDst, " ");
      pDst++;
   }

   for (i = 0; i < elements; i++)
   {
      if ((i != 0) && (nFormat == DSL_ARRAY_FORMAT_HEX))
      {
         ret = sprintf(pDst, " ");
         pDst ++;
      }

      if (nFormat == DSL_ARRAY_FORMAT_HEX)
      {
         switch (nSrcElementSize)
         {
         case 1:
            ret = sprintf(pDst, "%02X", ((DSL_uint8_t*)pSrc)[i]);
            break;
         case 2:
            ret = sprintf(pDst, "%04X", ((DSL_uint16_t*)pSrc)[i]);
            break;
         case 4:
            ret = sprintf(pDst, "%08X", ((DSL_uint32_t*)pSrc)[i]);
            break;
         default:
            ret = sprintf(pDst, "xx");
            break;
         }

         if(ret > 0)
         {
            pDst += ret;
         }
      }
      else
      {
         switch (nSrcElementSize)
         {
         case 1:
            nVal = (DSL_uint32_t)(((DSL_uint8_t*)pSrc)[i]);
            break;
         case 2:
            nVal = (DSL_uint32_t)(((DSL_uint16_t*)pSrc)[i]);
            break;
         case 4:
            nVal = (DSL_uint32_t)(((DSL_uint32_t*)pSrc)[i]);
            break;
         default:
            nVal = 0;
            break;
         }

         for (j = 0; j < nSrcElementSize; j++)
         {
            c = (DSL_char_t)(nVal >> j);
            if (isprint(c) != 0)
            {
               ret = sprintf(pDst, "%c", c);
               pDst += ret;
            }
         }
      }
   }

   if (nFormat == DSL_ARRAY_FORMAT_HEX)
   {
      ret = sprintf(pDst, " ");
   }
}

static DSL_int_t DSL_DRV_PROC_AvsarVerShow(struct seq_file* sfile)
{
   DSL_VersionInformation_t pData;
   DSL_int_t devNum = 0;
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;
   
   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return 0;
   }

   devNum = pDevEntry->devNum;

   if (ifxDevices[devNum].pContext != DSL_NULL)
   {
      /* Get Version Information*/
      DSL_DRV_VersionInformationGet(ifxDevices[devNum].pContext, &pData);
   }
   else
   {
      strcpy(pData.data.DSL_DriverVersionApi, "n/a");
      strcpy(pData.data.DSL_ChipSetFWVersion, "n/a");
      strcpy(pData.data.DSL_ChipSetHWVersion, "n/a");
      strcpy(pData.data.DSL_ChipSetType, "n/a");
      strcpy(pData.data.DSL_DriverVersionMeiBsp, "n/a");
   }

   seq_printf(sfile, "ATM Driver version:[%s]" DSL_DRV_CRLF, pData.data.DSL_DriverVersionApi);
   seq_printf(sfile, "DSL HAL version:[%s]" DSL_DRV_CRLF, "n/a");
   seq_printf(sfile, "SAR HAL version:[%s]" DSL_DRV_CRLF, "n/a");
   seq_printf(sfile, "PDSP Firmware version:[%s]" DSL_DRV_CRLF, "n/a");
   seq_printf(sfile, "Chipset ID:[%s]" DSL_DRV_CRLF, pData.data.DSL_ChipSetType);

   pDevEntry->SecPrivateData.bEnd = DSL_TRUE;
  
   return 0;
}

static DSL_int_t DSL_DRV_PROC_AvsarModemTrainingShow(struct seq_file* sfile)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;   
   DSL_int_t devNum = 0;
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;
   
   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return 0;
   }

   devNum = pDevEntry->devNum;

   if (ifxDevices[devNum].pContext != DSL_NULL)
   {
      /* Get Line State value*/
      DSL_CTX_READ_SCALAR(ifxDevices[devNum].pContext, nErrCode, nLineState, nCurrentState);

      if (nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC ||
          nCurrentState == DSL_LINESTATE_SHOWTIME_NO_SYNC)
      {
         seq_printf(sfile, "SHOWTIME"DSL_DRV_CRLF);
         seq_printf(sfile, "%s"DSL_DRV_CRLF, "n/a");
         seq_printf(sfile, "failTrains=%s"DSL_DRV_CRLF, "n/a");
      }
   }

   pDevEntry->SecPrivateData.bEnd = DSL_TRUE;
   
   return (DSL_int_t)nErrCode;
}

static DSL_int_t DSL_DRV_PROC_AvsarModemStatsShow(struct seq_file* sfile)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   union
   {
      DSL_G997_ChannelStatus_t channelStatus;
      DSL_G997_LineStatus_t    lineStatus;
      DSL_G997_LineInventory_t lineInventory;
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
      DSL_PM_LineInitCountersTotal_t pmTotalLineInitCounters;
#endif /* INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
      DSL_PM_ChannelCountersTotal_t  pmTotalChannelCounters;
#endif /* INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/
      DSL_PM_DataPathCountersTotal_t pmTotalDataPathCounters;
   } pData;
   DSL_char_t sBuf[256];
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;   
   DSL_int_t devNum = 0;
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;
   
   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return 0;
   }

   devNum = pDevEntry->devNum;

   if (ifxDevices[devNum].pContext != DSL_NULL)
   {
      seq_printf(sfile,"DSL Modem Statistics" DSL_DRV_CRLF
                       "--------------------------------"DSL_DRV_CRLF
                       "[DSL Modem Stats]"DSL_DRV_CRLF);

      /* Get US channel status*/
      pData.channelStatus.nChannel = 0;
      pData.channelStatus.nDirection = DSL_UPSTREAM;
      nErrCode = DSL_DRV_G997_ChannelStatusGet(ifxDevices[devNum].pContext, &(pData.channelStatus));
 
      /* Print US Data Rate value*/
      seq_printf(sfile, "    US Connection Rate        :%7s",
         nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa((DSL_int_t)(pData.channelStatus.data.ActualDataRate/1000)) : "n/a");
      
      /* Get DS channel status*/
      pData.channelStatus.nChannel = 0;
      pData.channelStatus.nDirection = DSL_DOWNSTREAM;
      nErrCode = DSL_DRV_G997_ChannelStatusGet(ifxDevices[devNum].pContext, &(pData.channelStatus));

      /* Print DS Data Rate value*/
      seq_printf(sfile, "    DS Connection Rate        :%7s"DSL_DRV_CRLF,
         nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa((DSL_int_t)(pData.channelStatus.data.ActualDataRate/1000)) : "n/a");

      /* Get DS line status*/
      pData.lineStatus.nDeltDataType = DSL_DELT_DATA_SHOWTIME;
      pData.lineStatus.nDirection = DSL_DOWNSTREAM;
      nErrCode = DSL_DRV_G997_LineStatusGet(ifxDevices[devNum].pContext, &(pData.lineStatus));

      /* Print DS Line Attenuation value*/
      seq_printf(sfile, "    DS Line Attenuation       :%7s",
          nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa(pData.lineStatus.data.LATN) : "n/a");

      /* Print DS SNR Margin value*/
      seq_printf(sfile, "    DS Margin                 :%7s"DSL_DRV_CRLF,
          nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa(pData.lineStatus.data.SNR) : "n/a");

      /* Get US line status*/
      pData.lineStatus.nDeltDataType = DSL_DELT_DATA_SHOWTIME;
      pData.lineStatus.nDirection = DSL_UPSTREAM;
      nErrCode = DSL_DRV_G997_LineStatusGet(ifxDevices[devNum].pContext, &(pData.lineStatus));

      /* Print US Line Attenuation value*/
      seq_printf(sfile, "    US Line Attenuation       :%7s",
          nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa(pData.lineStatus.data.LATN) : "n/a");

      /* Print US SNR Margin value*/
      seq_printf(sfile, "    US Margin                 :%7s"DSL_DRV_CRLF,
          nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa(pData.lineStatus.data.SNR) : "n/a");

      /* Print US Payload value, NOT SUPPORTED*/
      seq_printf(sfile, "    US Payload                :%7s", "n/a");
      /* Print DS Payload value, NOT SUPPORTED*/
      seq_printf(sfile, "    DS Payload                :%7s"DSL_DRV_CRLF, "n/a");

      /* Print US Superframe Cnt value, NOT SUPPORTED*/
      seq_printf(sfile, "    US Superframe Cnt         :%7s", "n/a");
      /* Print DS Superframe Cnt value, NOT SUPPORTED*/
      seq_printf(sfile, "    US Superframe Cnt         :%7s"DSL_DRV_CRLF, "n/a");

      /* Get US line status*/
      pData.lineStatus.nDeltDataType = DSL_DELT_DATA_SHOWTIME;
      pData.lineStatus.nDirection = DSL_UPSTREAM;
      nErrCode = DSL_DRV_G997_LineStatusGet(ifxDevices[devNum].pContext, &(pData.lineStatus));

      /* Print US Transmit Power value*/
      seq_printf(sfile, "    US Transmit Power         :%7s",
          nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa(pData.lineStatus.data.ACTATP) : "n/a");

      /* Get DS line status*/
      pData.lineStatus.nDeltDataType = DSL_DELT_DATA_SHOWTIME;
      pData.lineStatus.nDirection = DSL_DOWNSTREAM;
      nErrCode = DSL_DRV_G997_LineStatusGet(ifxDevices[devNum].pContext, &(pData.lineStatus));

      /* Print DS Transmit Power value*/
      seq_printf(sfile, "    DS Transmit Power         :%7s"DSL_DRV_CRLF,
          nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa(pData.lineStatus.data.ACTATP) : "n/a");

      /* Print LOS errors value, NOT SUPPORTED*/
      seq_printf(sfile, "    LOS errors                :%7s", "n/a");
      /* Print SEF errors value, NOT SUPPORTED*/
      seq_printf(sfile, "    SEF errors                :%7s"DSL_DRV_CRLF, "n/a");

      /* Print Frame mode value, NOT SUPPORTED*/
      seq_printf(sfile, "    Frame mode                :%7s", "n/a");
      /* Print Max Frame mode value, NOT SUPPORTED*/
      seq_printf(sfile, "    Max Frame mode            :%7s"DSL_DRV_CRLF, "n/a");

      /* Print Trained Path value, NOT SUPPORTED*/
      seq_printf(sfile, "    Trained Path              :%7s", "n/a");
      /* Print US Peak Cell Rate value, NOT SUPPORTED*/
      seq_printf(sfile, "    US Peak Cell Rate         :%7s"DSL_DRV_CRLF, "n/a");

      /* Print Trained Mode value, NOT SUPPORTED*/
      seq_printf(sfile, "    Trained Mode              :%7s", "n/a");
      /* Print Selected Mode value, NOT SUPPORTED*/
      seq_printf(sfile, "    Selected Mode             :%7s"DSL_DRV_CRLF, "n/a");

      /* Print ATUC Vendor Code value, NOT SUPPORTED*/
      seq_printf(sfile, "    ATUC Vendor Code          :%7s", "n/a");
      /* Print ATUC Revision value, NOT SUPPORTED*/
      seq_printf(sfile, "    ATUC Revision             :%7s"DSL_DRV_CRLF, "n/a");

      /* Print Hybrid Selected value, NOT SUPPORTED*/
      seq_printf(sfile, "    Hybrid Selected           :%7s", "n/a");
      /* Print Trellis value, NOT SUPPORTED*/
      seq_printf(sfile, "    Trellis                   :%7s"DSL_DRV_CRLF, "n/a");

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
      /* Get Total Line Init counters*/
      memset(&(pData.pmTotalLineInitCounters), 0x0, sizeof(DSL_PM_LineInitCountersTotal_t));
      nErrCode = DSL_DRV_PM_LineInitCountersTotalGet(
                    ifxDevices[devNum].pContext,
                    &(pData.pmTotalLineInitCounters));
      /* Get Line State value*/
      DSL_CTX_READ_SCALAR(ifxDevices[devNum].pContext, nErrCode, nLineState, nCurrentState);

      /* Print Showtime Count value*/
      seq_printf(sfile, "    Showtime Count            :%7s",
          (nErrCode == DSL_SUCCESS) && (nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC) ?
          DSL_DRV_Itoa(pData.pmTotalLineInitCounters.data.nFullInits - 
                       pData.pmTotalLineInitCounters.data.nFailedFullInits) : "n/a");
#else
      /* Print Showtime Count value*/
      seq_printf(sfile, "    Showtime Count            :%7s", "n/a");
#endif

      /* Get DS line status*/
      pData.lineStatus.nDeltDataType = DSL_DELT_DATA_SHOWTIME;
      pData.lineStatus.nDirection = DSL_DOWNSTREAM;
      nErrCode = DSL_DRV_G997_LineStatusGet(ifxDevices[devNum].pContext, &(pData.lineStatus));

      /* Print DS Max Attainable Bit Rate value*/
      seq_printf(sfile, "    DS Max Attainable Bit Rate:%7s kbps"DSL_DRV_CRLF,
                nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa(pData.lineStatus.data.ATTNDR/1000) : "n/a");

      /* Print BitSwap value, NOT SUPPORTED*/
      seq_printf(sfile, "    BitSwap                   :%7s", "n/a");

      /* Get US line status*/
      pData.lineStatus.nDeltDataType = DSL_DELT_DATA_SHOWTIME;
      pData.lineStatus.nDirection = DSL_UPSTREAM;
      nErrCode = DSL_DRV_G997_LineStatusGet(ifxDevices[devNum].pContext, &(pData.lineStatus));

      /* Print US Max Attainable Bit Rate value*/
      seq_printf(sfile, "    US Max Attainable Bit Rate:%7s kbps"DSL_DRV_CRLF,
          nErrCode == DSL_SUCCESS ? DSL_DRV_Itoa(pData.lineStatus.data.ATTNDR/1000) : "n/a");

      /* Print Annex value, NOT SUPPORTED*/
      seq_printf(sfile, "    Annex                     :%7s", "n/a");
      /* Print psd_mask_qualifier value, NOT SUPPORTED*/
      seq_printf(sfile, "    psd_mask_qualifier        :%7s"DSL_DRV_CRLF, "n/a");

      /* Get FE Line Inventory Information*/
      memset(&(pData.lineInventory), 0x0, sizeof(DSL_G997_LineInventory_t));
      pData.lineInventory.nDirection = DSL_FAR_END;
      nErrCode = DSL_DRV_G997_LineInventoryGet(ifxDevices[devNum].pContext, &(pData.lineInventory));

      /* Get G994 Vendor ID in the HEX format*/
      DSL_DRV_ArraySPrintF(sBuf, pData.lineInventory.data.G994VendorID,
         sizeof(pData.lineInventory.data.G994VendorID),
         sizeof(pData.lineInventory.data.G994VendorID[0]),
         DSL_ARRAY_FORMAT_HEX);

      /* Print ATUC ghsVid value, NOT SUPPORTED*/
      seq_printf(sfile, "    ATUC ghsVid: %s"DSL_DRV_CRLF,
          (nErrCode == DSL_SUCCESS) && (nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC) ? sBuf : " n/a");

      /* Print T1413Vid value, NOT SUPPORTED*/
      seq_printf(sfile, "    T1413Vid: %7s", "n/a");
      /* Print T1413Rev value, NOT SUPPORTED*/
      seq_printf(sfile, "    T1413Vid: %7s", "n/a");
      /* Print T1413Rev value, NOT SUPPORTED*/
      seq_printf(sfile, "    VendorRev: %7s", "n/a"DSL_DRV_CRLF);

      /* Get NE Line Inventory Information*/
      memset(&(pData.lineInventory), 0x0, sizeof(DSL_G997_LineInventory_t));
      pData.lineInventory.nDirection = DSL_NEAR_END;
      nErrCode = DSL_DRV_G997_LineInventoryGet(ifxDevices[devNum].pContext, &(pData.lineInventory));

      /* Get G994 Vendor ID in the HEX format*/
      DSL_DRV_ArraySPrintF(sBuf, pData.lineInventory.data.G994VendorID,
         sizeof(pData.lineInventory.data.G994VendorID),
         sizeof(pData.lineInventory.data.G994VendorID[0]),
         DSL_ARRAY_FORMAT_HEX);

      /* Print ATUR ghsVid value, NOT SUPPORTED*/
      seq_printf(sfile, "    ATUR ghsVid: %s"DSL_DRV_CRLF,
          (nErrCode == DSL_SUCCESS) ? sBuf : " n/a");

      /* Print T1413Vid value, NOT SUPPORTED*/
      seq_printf(sfile, "    T1413Vid: %7s", "n/a");
      /* Print T1413Rev value, NOT SUPPORTED*/
      seq_printf(sfile, "    T1413Vid: %7s", "n/a");
      /* Print T1413Rev value, NOT SUPPORTED*/
      seq_printf(sfile, "    VendorRev: %7s", "n/a"DSL_DRV_CRLF);

      /*
         PM statistics
      */
      seq_printf(sfile, DSL_DRV_CRLF);      
      seq_printf(sfile, "    [Upstream (TX) Interleave path]"DSL_DRV_CRLF);

#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
      /* Get PM NE Total Channel Counters*/
      memset(&(pData.pmTotalChannelCounters), 0x0, sizeof(DSL_PM_ChannelCountersTotal_t));
      pData.pmTotalChannelCounters.nChannel = 0;
      pData.pmTotalChannelCounters.nDirection = DSL_NEAR_END;
      nErrCode = DSL_DRV_PM_ChannelCountersTotalGet(
                    ifxDevices[devNum].pContext,
                    &(pData.pmTotalChannelCounters));

      /* Print CRC value*/
      seq_printf(sfile, "    CRC: %7s",
          nErrCode == DSL_SUCCESS ?
          DSL_DRV_Itoa(pData.pmTotalChannelCounters.data.nCodeViolations) : "n/a");
      /* Print FEC value*/
      seq_printf(sfile, "    FEC: %7s",
          nErrCode == DSL_SUCCESS ?
          DSL_DRV_Itoa(pData.pmTotalChannelCounters.data.nFEC) : "n/a");
#else
      /* Print CRC value*/
      seq_printf(sfile, "    CRC: %7s", "n/a");
      /* Print FEC value*/
      seq_printf(sfile, "    FEC: %7s", "n/a");
#endif /* INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

      /* Print NCD value, NOT SUPPORTED*/
      seq_printf(sfile, "    NCD: %7s", "n/a"DSL_DRV_CRLF);
      /* Print LCD value, NOT SUPPORTED*/
      seq_printf(sfile, "    LCD: %7s", "n/a");
      /* Print HEC value, NOT SUPPORTED*/
      seq_printf(sfile, "    HEC: %7s"DSL_DRV_CRLF, "n/a");

      seq_printf(sfile, DSL_DRV_CRLF);      
      seq_printf(sfile, "    [Downstream (RX) Interleave path]"DSL_DRV_CRLF);

#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
      /* Get PM FE Total Channel Counters*/
      memset(&(pData.pmTotalChannelCounters), 0x0, sizeof(DSL_PM_ChannelCountersTotal_t));
      pData.pmTotalChannelCounters.nChannel = 0;
      pData.pmTotalChannelCounters.nDirection = DSL_FAR_END;
      nErrCode = DSL_DRV_PM_ChannelCountersTotalGet(
                    ifxDevices[devNum].pContext,
                    &(pData.pmTotalChannelCounters));

      /* Print CRC value*/
      seq_printf(sfile, "    CRC: %7s",
          nErrCode == DSL_SUCCESS ?
          DSL_DRV_Itoa(pData.pmTotalChannelCounters.data.nCodeViolations) : "n/a");
      /* Print FEC value*/
      seq_printf(sfile, "    FEC: %7s",
          nErrCode == DSL_SUCCESS ?
          DSL_DRV_Itoa(pData.pmTotalChannelCounters.data.nFEC) : "n/a");
#else
      /* Print CRC value*/
      seq_printf(sfile, "    CRC: %7s", "n/a");
      /* Print FEC value*/
      seq_printf(sfile, "    FEC: %7s", "n/a");
#endif
      /* Print NCD value, NOT SUPPORTED*/
      seq_printf(sfile, "    NCD: %7s", "n/a"DSL_DRV_CRLF);
      /* Print LCD value, NOT SUPPORTED*/
      seq_printf(sfile, "    LCD: %7s", "n/a");
      /* Print HEC value, NOT SUPPORTED*/
      seq_printf(sfile, "    HEC: %7s"DSL_DRV_CRLF,"n/a");

      seq_printf(sfile, DSL_DRV_CRLF);      
      seq_printf(sfile, "    [Upstream (TX) Fast path]"DSL_DRV_CRLF);
      /* Print CRC value, NOT SUPPORTED*/
      seq_printf(sfile, "    CRC: %7s","n/a");
      /* Print FEC value, NOT SUPPORTED*/
      seq_printf(sfile, "    FEC: %7s","n/a");
      /* Print NCD value, NOT SUPPORTED*/
      seq_printf(sfile, "    NCD: %7s","n/a"DSL_DRV_CRLF);
      /* Print LCD value, NOT SUPPORTED*/
      seq_printf(sfile, "    LCD: %7s", "n/a");
      /* Print HEC value, NOT SUPPORTED*/
      seq_printf(sfile, "    HEC: %7s"DSL_DRV_CRLF, "n/a");

      seq_printf(sfile, DSL_DRV_CRLF);      
      seq_printf(sfile, "    [Downstream (RX) Fast path]"DSL_DRV_CRLF);
      /* Print CRC value, NOT SUPPORTED*/
      seq_printf(sfile, "    CRC: %7s","n/a");
      /* Print FEC value, NOT SUPPORTED*/
      seq_printf(sfile, "    FEC: %7s","n/a");
      /* Print NCD value, NOT SUPPORTED*/
      seq_printf(sfile, "    NCD: %7s","n/a"DSL_DRV_CRLF);
      /* Print LCD value, NOT SUPPORTED*/
      seq_printf(sfile, "    LCD: %7s", "n/a");
      /* Print HEC value, NOT SUPPORTED*/
      seq_printf(sfile, "    HEC: %7s"DSL_DRV_CRLF, "n/a");

      seq_printf(sfile, DSL_DRV_CRLF);
      seq_printf(sfile, "[ATM Stats]"DSL_DRV_CRLF);
      seq_printf(sfile, "    [Upstream/TX]"DSL_DRV_CRLF);

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
      /* Get PM NE Total Data Path Counters*/
      memset(&(pData.pmTotalDataPathCounters), 0x0, sizeof(DSL_PM_DataPathCountersTotal_t));
      pData.pmTotalDataPathCounters.nChannel = 0;
      pData.pmTotalDataPathCounters.nDirection = DSL_NEAR_END;
      nErrCode = DSL_DRV_PM_DataPathCountersTotalGet(
                    ifxDevices[devNum].pContext,
                    &(pData.pmTotalDataPathCounters));

      /* Print Good Cell Cnt value*/
      seq_printf(sfile, "    Good Cell Cnt            : %7s"DSL_DRV_CRLF,
          nErrCode == DSL_SUCCESS ?
          DSL_DRV_Itoa(pData.pmTotalDataPathCounters.data.nUserTotalCells) : "n/a");
      /* Print Idle Cell Cnt value*/
      seq_printf(sfile, "    Idle Cell Cnt            : %7s"DSL_DRV_CRLF,
          nErrCode == DSL_SUCCESS ?
          DSL_DRV_Itoa(pData.pmTotalDataPathCounters.data.nIBE) : "n/a");
#else
      /* Print Good Cell Cnt value*/
      seq_printf(sfile, "    Good Cell Cnt            : %7s"DSL_DRV_CRLF, "n/a");
      /* Print Idle Cell Cnt value*/
      seq_printf(sfile, "    Idle Cell Cnt            : %7s"DSL_DRV_CRLF, "n/a");
#endif
      seq_printf(sfile, DSL_DRV_CRLF);
      seq_printf(sfile, "    [Downstream/RX]"DSL_DRV_CRLF);

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
      memset(&(pData.pmTotalDataPathCounters), 0x0, sizeof(DSL_PM_DataPathCountersTotal_t));
      pData.pmTotalDataPathCounters.nChannel = 0;
      pData.pmTotalDataPathCounters.nDirection = DSL_FAR_END;
      nErrCode = DSL_DRV_PM_DataPathCountersTotalGet(
                    ifxDevices[devNum].pContext,
                    &(pData.pmTotalDataPathCounters));

      /* Print Good Cell Cnt value*/
      seq_printf(sfile, "    Good Cell Cnt            : %7s"DSL_DRV_CRLF,
          nErrCode == DSL_SUCCESS ?
          DSL_DRV_Itoa(pData.pmTotalDataPathCounters.data.nUserTotalCells) : "n/a");
      /* Print Idle Cell Cnt value*/
      seq_printf(sfile, "    Idle Cell Cnt            : %7s"DSL_DRV_CRLF,
          nErrCode == DSL_SUCCESS ?
          DSL_DRV_Itoa(pData.pmTotalDataPathCounters.data.nIBE) : "n/a");
      /* Print Bad Hec Cell Cnt value*/
      seq_printf(sfile, "    Bad Hec Cell Cnt         : %7s"DSL_DRV_CRLF,
          nErrCode == DSL_SUCCESS ?
          DSL_DRV_Itoa(pData.pmTotalDataPathCounters.data.nHEC) : "n/a");
#else
      /* Print Good Cell Cnt value*/
      seq_printf(sfile, "    Good Cell Cnt            : %7s"DSL_DRV_CRLF, "n/a");
      /* Print Idle Cell Cnt value*/
      seq_printf(sfile, "    Idle Cell Cnt            : %7s"DSL_DRV_CRLF, "n/a");
      /* Print Bad Hec Cell Cnt value*/
      seq_printf(sfile, "    Bad Hec Cell Cnt         : %7s"DSL_DRV_CRLF, "n/a");
#endif
      /* Print Overflow Dropped Cell Cnt value, NOT SUPPORTED*/
      seq_printf(sfile, "    Overflow Dropped Cell Cnt: %7s"DSL_DRV_CRLF, "n/a");

      seq_printf(sfile, DSL_DRV_CRLF);
      seq_printf(sfile, "[SAR AAL5 Stats]"DSL_DRV_CRLF);
      /* Print Tx PDU's Counts value, NOT SUPPORTED*/
      seq_printf(sfile, "    Tx PDU's             : %7s"DSL_DRV_CRLF, "n/a");
      /* Print Rx PDU's Counts value, NOT SUPPORTED*/
      seq_printf(sfile, "    Rx PDU's             : %7s"DSL_DRV_CRLF, "n/a");
      /* Print Tx Total Bytes Counts value, NOT SUPPORTED*/
      seq_printf(sfile, "    Tx Total Bytes       : %7s"DSL_DRV_CRLF, "n/a");
      /* Print Rx Total Bytes Counts value, NOT SUPPORTED*/
      seq_printf(sfile, "    Rx Total Bytes       : %7s"DSL_DRV_CRLF, "n/a");
      /* Print Tx Total Error Counts value, NOT SUPPORTED*/
      seq_printf(sfile, "    Tx Total Error Counts: %7s"DSL_DRV_CRLF, "n/a");
      /* Print Rx Total Error Counts value, NOT SUPPORTED*/
      seq_printf(sfile, "    Rx Total Error Counts: %7s"DSL_DRV_CRLF, "n/a");

      seq_printf(sfile, DSL_DRV_CRLF);
      seq_printf(sfile, "[OAM Stats]"DSL_DRV_CRLF);
      /* Print Near End F5 Loop Back Count value, NOT SUPPORTED*/
      seq_printf(sfile, "    Near End F5 Loop Back Count: %7s"DSL_DRV_CRLF, "n/a");
      /* Print Near End F4 Loop Back Count value, NOT SUPPORTED*/
      seq_printf(sfile, "    Near End F4 Loop Back Count: %7s"DSL_DRV_CRLF, "n/a");
      /* Print Far End F5 Loop Back Count value, NOT SUPPORTED*/
      seq_printf(sfile, "    Far End F5 Loop Back Count : %7s"DSL_DRV_CRLF, "n/a");
      /* Print Fa End F4 Loop Back Count value, NOT SUPPORTED*/
      seq_printf(sfile, "    Far End F4 Loop Back Count : %7s"DSL_DRV_CRLF, "n/a");

      seq_printf(sfile, DSL_DRV_CRLF);
   }

   pDevEntry->SecPrivateData.bEnd = DSL_TRUE;
  
   return 0;
}

#ifdef INCLUDE_DSL_G997_PER_TONE
static DSL_void_t* DSL_DRV_PROC_BitAllocationTableDataGet(struct seq_file* sfile)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t devNum = 0;
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;
   DSL_void_t *pBlock = DSL_NULL;
   DSL_G997_BitAllocationNsc_t *pData = DSL_NULL;
   
   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return 0;
   }

   devNum = pDevEntry->devNum;

   if (ifxDevices[devNum].pContext != DSL_NULL)
   {
      /* Allocate data for US and DS BAT*/
      pBlock = kmalloc(2 * sizeof(DSL_G997_BitAllocationNsc_t), GFP_KERNEL);

      if (!pBlock)
      {
         return DSL_NULL;
      }

      pData = (DSL_G997_BitAllocationNsc_t*)pBlock;

      /* Get US BAT*/
      pData->nDirection = DSL_UPSTREAM;
      nErrCode = DSL_DRV_G997_BitAllocationNSCGet(ifxDevices[devNum].pContext, pData);
      if (nErrCode != DSL_SUCCESS)
      {
         kfree(pBlock);
         return DSL_NULL;
      }

      pData++;
      
      /* Get DS BAT*/
      pData->nDirection = DSL_DOWNSTREAM;
      nErrCode = DSL_DRV_G997_BitAllocationNSCGet(ifxDevices[devNum].pContext, pData);
      if (nErrCode != DSL_SUCCESS)
      {
         kfree(pBlock);      
         return DSL_NULL;
      }
   }
   else
   {
      return DSL_NULL;
   }
   
   return pBlock;
}

static DSL_int_t DSL_DRV_PROC_BitAllocationTableShow(struct seq_file* sfile)
{
   DSL_uint32_t i = 0;
   DSL_int_t devNum = 0;
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;
   DSL_G997_BitAllocationNsc_t *pBAT;
   
   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return 0;
   }

   devNum = pDevEntry->devNum;

   if (ifxDevices[devNum].pContext != DSL_NULL)
   {
      if (pDevEntry->SecPrivateData.pPrivateData && pDevEntry->SecPrivateData.pPrintData)
      {
         pBAT = (DSL_G997_BitAllocationNsc_t*)pDevEntry->SecPrivateData.pPrintData;
         
         if (pDevEntry->SecPrivateData.v == 0)
         {
            seq_printf(sfile, "DSL Modem %s Bit Allocation:"DSL_DRV_CRLF,
            pBAT->nDirection == DSL_UPSTREAM ? "US" : "DS");
         }

         for (i = pDevEntry->SecPrivateData.v*256;
             (i < ((pDevEntry->SecPrivateData.v+1)*256)) && (i < pBAT->data.bitAllocationNsc.nNumData) &&
             (i < DSL_MAX_NSC);
             i++)
         {
            seq_printf(sfile, "%02X ", (DSL_int_t)pBAT->data.bitAllocationNsc.nNSCData[i]);
            if ( !((i+1)%16))
            {
               seq_printf(sfile, DSL_DRV_CRLF);
            }
         }

         pDevEntry->SecPrivateData.v++;
         
         if (pDevEntry->SecPrivateData.v*256 >= pBAT->data.bitAllocationNsc.nNumData)
         {
            if (pDevEntry->SecPrivateData.pPrintData != pDevEntry->SecPrivateData.pPrivateData)
            {
               /* Sequence end*/
               pDevEntry->SecPrivateData.bEnd = DSL_TRUE;
               seq_printf(sfile, DSL_DRV_CRLF);
            }
            else
            {
               /* Move to the DS data*/
               pDevEntry->SecPrivateData.pPrintData = (DSL_void_t*)(++pBAT);

               /* reset iteration number*/
               pDevEntry->SecPrivateData.v = 0;
               
               seq_printf(sfile, DSL_DRV_CRLF);
            }
         }
      }
      else
      {
         seq_printf(sfile, "DSL Modem US Bit Allocation:"DSL_DRV_CRLF);
         seq_printf(sfile, "n/a"DSL_DRV_CRLF);
         seq_printf(sfile, "DSL Modem US Bit Allocation:"DSL_DRV_CRLF);
         seq_printf(sfile, "n/a"DSL_DRV_CRLF);
         seq_printf(sfile, DSL_DRV_CRLF);
         /* Sequence end*/
         pDevEntry->SecPrivateData.bEnd = DSL_TRUE;            
      }
   }
  
   return 0;
}

static DSL_void_t* DSL_DRV_PROC_SnrDsAllocationTableDataGet(struct seq_file* sfile)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t devNum = 0;
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;
   DSL_void_t *pBlock = DSL_NULL;
   DSL_G997_SnrAllocationNsc_t *pData = DSL_NULL;
   
   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return 0;
   }

   devNum = pDevEntry->devNum;

   if (ifxDevices[devNum].pContext != DSL_NULL)
   {
      /* Allocate data for DS*/
      pBlock = kmalloc(sizeof(DSL_G997_SnrAllocationNsc_t), GFP_KERNEL);

      if (!pBlock)
      {
         return DSL_NULL;
      }

      pData = (DSL_G997_SnrAllocationNsc_t*)pBlock;

      /* Get DS SNRs*/
      pData->nDirection = DSL_DOWNSTREAM;
      nErrCode = DSL_DRV_G997_SnrAllocationNscGet(ifxDevices[devNum].pContext, pData);
      if (nErrCode != DSL_SUCCESS)
      {
         kfree(pBlock);
         return DSL_NULL;
      }
   }
   else
   {
      return DSL_NULL;
   }
   
   return pBlock;
}

static DSL_int_t DSL_DRV_PROC_SnrDsAllocationTableShow(struct seq_file* sfile)
{
   DSL_uint32_t i = 0;
   DSL_int_t devNum = 0;
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;
   DSL_G997_SnrAllocationNsc_t *pSNR;
   
   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return 0;
   }

   devNum = pDevEntry->devNum;

   if (ifxDevices[devNum].pContext != DSL_NULL)
   {
      if (pDevEntry->SecPrivateData.pPrivateData && pDevEntry->SecPrivateData.pPrintData)
      {
         pSNR = (DSL_G997_SnrAllocationNsc_t*)pDevEntry->SecPrivateData.pPrintData;
         
         if (pDevEntry->SecPrivateData.v == 0)
         {
            seq_printf(sfile, "SNRpsds:"DSL_DRV_CRLF);
         }

         for (i = pDevEntry->SecPrivateData.v*256;
             (i < ((pDevEntry->SecPrivateData.v+1)*256)) && (i < pSNR->data.snrAllocationNsc.nNumData) &&
             (i < DSL_MAX_NSC);
             i++)
         {
            seq_printf(sfile, "%d ", (DSL_int_t)pSNR->data.snrAllocationNsc.nNSCData[i]);
            if ( !((i+1)%16))
            {
               seq_printf(sfile, DSL_DRV_CRLF);
            }
         }

         pDevEntry->SecPrivateData.v++;
         
         if (pDevEntry->SecPrivateData.v*256 >= pSNR->data.snrAllocationNsc.nNumData)
         {
            /* Sequence end*/
            pDevEntry->SecPrivateData.bEnd = DSL_TRUE;
            seq_printf(sfile, DSL_DRV_CRLF);
         }
      }
      else
      {
         seq_printf(sfile, "SNRpsds:"DSL_DRV_CRLF);
         seq_printf(sfile, "n/a"DSL_DRV_CRLF);
         seq_printf(sfile, DSL_DRV_CRLF);
         /* Sequence end*/
         pDevEntry->SecPrivateData.bEnd = DSL_TRUE;            
      }
   }
  
   return 0;
}
#endif /* INCLUDE_DSL_G997_PER_TONE*/

#ifdef INCLUDE_DSL_DELT
static DSL_void_t* DSL_DRV_PROC_QlnDsShowtimeDataGet(struct seq_file* sfile)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t devNum = 0;
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;
   DSL_void_t *pBlock = DSL_NULL;
   DSL_G997_DeltQln_t *pData = DSL_NULL;
   
   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return 0;
   }

   devNum = pDevEntry->devNum;

   if (ifxDevices[devNum].pContext != DSL_NULL)
   {
      /* Allocate data for DS*/
      pBlock = kmalloc(sizeof(DSL_G997_DeltQln_t), GFP_KERNEL);

      if (!pBlock)
      {
         return DSL_NULL;
      }

      pData = (DSL_G997_DeltQln_t*)pBlock;

      /* Get DS SNRs*/
      pData->nDirection = DSL_DOWNSTREAM;
      pData->nDeltDataType = DSL_DELT_DATA_SHOWTIME;
      nErrCode = DSL_DRV_G997_DeltQLNGet(ifxDevices[devNum].pContext, pData);
      if (nErrCode != DSL_SUCCESS)
      {
         kfree(pBlock);
         return DSL_NULL;
      }
   }
   else
   {
      return DSL_NULL;
   }
   
   return pBlock;
}

static DSL_int_t DSL_DRV_PROC_QlnDsShowtimeShow(struct seq_file* sfile)
{
   DSL_uint32_t i = 0;
   DSL_int_t devNum = 0;
   DSL_DRV_PROC_EntryData_t *pDevEntry = (DSL_DRV_PROC_EntryData_t*)sfile->private;
   DSL_G997_DeltQln_t *pQLN;
   
   if (pDevEntry == NULL)
   {
      printk("DSL_DRV: cannot find device proc entry data!" DSL_DRV_CRLF);   
      return 0;
   }

   devNum = pDevEntry->devNum;

   if (ifxDevices[devNum].pContext != DSL_NULL)
   {
      if (pDevEntry->SecPrivateData.pPrivateData && pDevEntry->SecPrivateData.pPrintData)
      {
         pQLN = (DSL_G997_DeltQln_t*)pDevEntry->SecPrivateData.pPrintData;
         
         if (pDevEntry->SecPrivateData.v == 0)
         {
            seq_printf(sfile, "QLNpsds:"DSL_DRV_CRLF);
         }

         for (i = pDevEntry->SecPrivateData.v*256;
             (i < ((pDevEntry->SecPrivateData.v+1)*256)) && (i < pQLN->data.deltQln.nNumData) &&
             (i < DSL_MAX_NSC); i++)
         {
            seq_printf(sfile, "%d ", (DSL_int_t)pQLN->data.deltQln.nNSCData[i]);
            if ( !((i+1)%16))
            {
               seq_printf(sfile, DSL_DRV_CRLF);
            }
         }

         pDevEntry->SecPrivateData.v++;
         
         if (pDevEntry->SecPrivateData.v*256 >= pQLN->data.deltQln.nNumData)
         {
            /* Sequence end*/
            pDevEntry->SecPrivateData.bEnd = DSL_TRUE;
            seq_printf(sfile, DSL_DRV_CRLF);
         }
      }
      else
      {
         seq_printf(sfile, "QLNpsds:"DSL_DRV_CRLF);
         seq_printf(sfile, "n/a"DSL_DRV_CRLF);
         seq_printf(sfile, DSL_DRV_CRLF);
         /* Sequence end*/
         pDevEntry->SecPrivateData.bEnd = DSL_TRUE;            
      }
   }
  
   return 0;
}
#endif /* INCLUDE_DSL_DELT*/

#ifdef __cplusplus
}
#endif

#endif /* INCLUDE_DSL_CPE_PROCFILES*/
