/******************************************************************************

                               Copyright (c) 2007
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

/** \file
   Danube specific DSL CLI, access function implementation
*/

#include "dsl_cpe_control.h"

#if defined (INCLUDE_DSL_CPE_API_DANUBE) || defined(INCLUDE_DSL_CPE_API_AMAZON_SE) || \
    defined (INCLUDE_DSL_CPE_API_AMAZON_S)

#include "dsl_cpe_cli.h"
#include "dsl_cpe_debug.h"
#include "drv_dsl_cpe_api.h"
#include "drv_dsl_cpe_api_ioctl.h"
#include "drv_dsl_cpe_cmv_danube.h"

#ifdef INCLUDE_DSL_CPE_CLI_SUPPORT

#undef DSL_CCA_DBG_BLOCK
#define DSL_CCA_DBG_BLOCK DSL_CCA_DBG_CLI

extern const char *sFailureReturn;

/* for debugging: */
#ifdef DSL_CLI_LOCAL
#undef DSL_CLI_LOCAL
#endif
#if 0
#define DSL_CLI_LOCAL
#else
#define DSL_CLI_LOCAL static
#endif

#ifndef _MKSTR_1
#define _MKSTR_1(x)    #x
#define _MKSTR(x)      _MKSTR_1(x)
#endif

#define DSL_CMV_MAX_LENGTH_TX     8
#define DSL_CMV_MAX_LENGTH_RX    10


static const DSL_char_t g_sCr[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_char_t sGroup[" _MKSTR((DSL_MAX_CMV_NAME_LENGTH - 1)) "]" DSL_CPE_CRLF
   "   optn | cnfg | cntl | stat | rate | plam | info | test" DSL_CPE_CRLF
   "- DSL_int_t nAddress (dec)" DSL_CPE_CRLF
   "- DSL_int_t nIndex (dec)" DSL_CPE_CRLF
   "- DSL_int_t nLength (dec)" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_int16_t nData[] (hex)" DSL_CPE_CRLF DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_DBG_CmvRead(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t nRet = 0;
   DSL_char_t sGroup[DSL_MAX_CMV_NAME_LENGTH] = {0};
   DSL_uint16_t nData[DSL_MAX_CMV_MSG_PAYLOAD_LENGTH] = {0};
   DSL_int_t nAddress = 0, nIndex = 0, nSize = 0, i = 0, j = 0;
   DSL_int_t nSendLen = 0, nMsgCnt = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 3, DSL_CLI_MIN) == DSL_FALSE)
   {
      return -1;
   }
   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 4, DSL_CLI_MAX) == DSL_FALSE)
   {
      return -1;
   }

   nRet = DSL_CPE_sscanf (pCommands, "%s %d %d %d", sGroup, &nAddress, &nIndex, &nSize);
   if ((nRet < 3) || (nSize < 0) || (nAddress < 0) || (nIndex < 0))
   {
      return -1;
   }
   else if (nRet == 3)
   {
      nSize = 1;
   }

   nMsgCnt = (nSize / DSL_MAX_CMV_MSG_PAYLOAD_LENGTH) + 1;

   for (i = 0; i < nMsgCnt; i++)
   {
      if (i == (nMsgCnt -1))
      {
         nSendLen = nSize - (i * DSL_MAX_CMV_MSG_PAYLOAD_LENGTH);
      }
      else
      {
         nSendLen = DSL_MAX_CMV_MSG_PAYLOAD_LENGTH;
      }

      nRet = DSL_CMV_Read (DSL_CPE_GetGlobalContext(), sGroup, nAddress, nIndex,
         nSendLen, &nData[0]);
      if (nRet < 0)
      {
         DSL_CPE_FPrintf (out, sFailureReturn, nRet);
      }
      else
      {
         nIndex += nSendLen;

         if (i == 0)
         {
            DSL_CPE_FPrintf (out, "nReturn=%d nData=\"" DSL_CPE_CRLF, nRet);
         }
         for (j = 0; j < nSendLen; j++)
         {
            DSL_CPE_FPrintf (out, "0x%04X ", nData[j]);
         }
         DSL_CPE_FPrintf (out, DSL_CPE_CRLF);
      }
   }
   DSL_CPE_FPrintf (out, "\"" DSL_CPE_CRLF);

   return 0;
}

static const DSL_char_t g_sCw[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_char_t sGroup[" _MKSTR((DSL_MAX_CMV_NAME_LENGTH - 1)) "]" DSL_CPE_CRLF
   "   optn | cnfg | cntl | stat | rate | plam | info | test" DSL_CPE_CRLF
   "- DSL_int_t nAddress (dec)" DSL_CPE_CRLF
   "- DSL_int_t nIndex (dec)" DSL_CPE_CRLF
   "- DSL_int16_t nData[] (hex)" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_DBG_CmvWrite(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t nRet = 0;
   DSL_char_t sGroup[DSL_MAX_CMV_NAME_LENGTH] = {0};
   DSL_uint16_t nMsgPayload[DSL_MAX_CMV_MSG_PAYLOAD_LENGTH] = {0};
   DSL_uint32_t nValue = 0;
   DSL_uint16_t nAddress = 0, nIndex = 0;
   DSL_int_t nSize = 1;
   DSL_char_t string[100] = { 0 };
   DSL_char_t seps[]   = " ";
   DSL_char_t *token;
   DSL_int_t i = 0, nParams = 0;

   nParams = DSL_MAX_CMV_MSG_PAYLOAD_LENGTH + 3;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 4, DSL_CLI_MIN) == DSL_FALSE)
   {
      return -1;
   }

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, nParams, DSL_CLI_MAX) == DSL_FALSE)
   {
      return -1;
   }

   strncpy (string, pCommands, sizeof(string)-1);
   string[sizeof(string)-1]=0;

   /* Get first token */
   token = strtok (string, seps);
   if (token != DSL_NULL)
   {
      for (i = 0; i < nParams; i++)
      {
         if (i == 0)
         {
            /* First token is group name string */
            DSL_CPE_sscanf (token, "%s", &sGroup);
         }
         else if (i == 1)
         {
            /* Second token is address */
            DSL_CPE_sscanf (token, "%hu", &nAddress);
         }
         else if (i == 2)
         {
            /* Third token is index */
            DSL_CPE_sscanf (token, "%hu", &nIndex);
         }
         else
         {
            /* Following tokens includes message data */
            DSL_CPE_sscanf (token, "%x", &nValue);
            nMsgPayload[i - 3] = (DSL_uint16_t)nValue;
         }

         /* Get next token */
         token = strtok(DSL_NULL, seps);

         /* Exit scanning if no further information is included */
         if (token == DSL_NULL)
         {
            break;
         }
      }
   }
   nSize = (i - 3) + 1;

   nRet = DSL_CMV_Write (DSL_CPE_GetGlobalContext(), sGroup, nAddress, nIndex,
      nSize, &(nMsgPayload[0]));
   if (nRet < 0)
   {
      DSL_CPE_FPrintf (out, sFailureReturn, nRet);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%d" DSL_CPE_CRLF, nRet);
   }

   return 0;
}

static const DSL_char_t g_sDms[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Input Parameter" DSL_CPE_CRLF
   "- DSL_int16_t nData[] (hex)" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_int16_t nData[] (hex)" DSL_CPE_CRLF DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_DBG_DeviceMessageSend(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_DeviceMessage_t pMsg;
   DSL_uint16_t msg[DSL_MAX_CMV_MSG_LENGTH] = {0};
   DSL_uint16_t nMsgSize = 0, nPayloadSize = 0, nPrintSize = 0;
   DSL_char_t string[100] = { 0 };
   DSL_char_t seps[]   = " ";
   DSL_char_t *token;
   DSL_uint32_t nValue = 0, nPayloadBitSize = 0;
   DSL_int_t nSize = 0, i = 0;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 4, DSL_CLI_MIN) == DSL_FALSE)
   {
      return -1;
   }

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, DSL_MAX_CMV_MSG_LENGTH, DSL_CLI_MAX) == DSL_FALSE)
   {
      return -1;
   }

   memset(&pMsg, 0, sizeof(pMsg));

   strncpy (string, pCommands, sizeof(string)-1);
   string[sizeof(string)-1]=0;

   /* Get number of included data words */
   nSize = 0;
   /* Get first token */
   token = strtok (string, seps);
   if (token != DSL_NULL)
   {
      for (i = 0; i < DSL_MAX_CMV_MSG_LENGTH; ++i)
      {
         /* Tokens includes message data */
         DSL_CPE_sscanf (token, "%x", &nValue);
         msg[i] = (DSL_uint16_t)nValue;

         /* Get next token */
         token = strtok(DSL_NULL, seps);

         /* Exit scanning if no further information is included */
         if (token == DSL_NULL)
         {
            break;
         }
      }
   }
   nSize = i + 1;

   if (nSize < DSL_CMV_HEADER_LENGTH)
   {
      return -1;
   }

#if 0
   nPayloadSize = msg[0] & 0xF;
   nMsgSize = (nPayloadSize + DSL_CMV_HEADER_LENGTH) * 2;

   /* Detect read/write */
   if ((msg[0] & 0x40) == 0x40)
   {
      /* write access */
      bWrite = DSL_TRUE;
      pMsg.data.nSizeTx = nMsgSize;
   }
   else
   {
      /* read access */
      bWrite = DSL_FALSE;
      pMsg.data.nSizeRx = nMsgSize;
      pMsg.data.nSizeTx = DSL_CMV_HEADER_LENGTH * 2;
   }
#else
   pMsg.data.nSizeTx = nSize * 2;
#endif

   pMsg.data.pMsg = (DSL_uint8_t *)&msg[0];

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_DBG_DEVICE_MESSAGE_SEND, (int) &pMsg);

   if ((ret < 0) && (pMsg.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, ret);
   }
   else
   {
      DSL_CPE_FPrintf (out, "nReturn=%d nSizeTx=%hu nSizeRx=%hu",
         pMsg.accessCtl.nReturn, pMsg.data.nSizeTx, pMsg.data.nSizeRx);

      if (pMsg.data.nSizeRx)
      {
         DSL_CPE_FPrintf (out, " nData=\"");
         nPrintSize = pMsg.data.nSizeRx - DSL_CMV_HEADER_LENGTH * 2;

         for (i = 0; i < (nPrintSize/2 + nPrintSize%2); i++)
         {
            DSL_CPE_FPrintf (out, "0x%04X ", msg[i + DSL_CMV_HEADER_LENGTH]);
         }

         DSL_CPE_FPrintf (out, " \"");
      }

      DSL_CPE_FPrintf (out, DSL_CPE_CRLF);
   }

   return 0;
}

static const DSL_char_t g_sVig[] =
#ifndef DSL_CPE_DEBUG_DISABLE
   "Long Form: %s" DSL_CPE_CRLF
   "Short Form: %s" DSL_CPE_CRLF
   DSL_CPE_CRLF
   "Output Parameter" DSL_CPE_CRLF
   "- DSL_Error_t nReturn" DSL_CPE_CRLF
   "- DSL_VersionInformation_t pData" DSL_CPE_CRLF
   DSL_CPE_CRLF "";
#else
   "";
#endif

DSL_CLI_LOCAL DSL_int_t DSL_CPE_CLI_VersionInformationGet(
   DSL_int_t fd,
   DSL_char_t *pCommands,
   DSL_CPE_File_t *out)
{
   DSL_Error_t ret = 0;
   DSL_VersionInformation_t pData;

   if (DSL_CPE_CLI_CheckParamNumber(pCommands, 0, DSL_CLI_EQUALS) == DSL_FALSE)
   {
      return -1;
   }

   ret = DSL_CPE_Ioctl (fd, DSL_FIO_VERSION_INFORMATION_GET, (int) &pData);

   if ((ret < 0) && (pData.accessCtl.nReturn < DSL_SUCCESS))
   {
      DSL_CPE_FPrintf (out, sFailureReturn, pData.accessCtl.nReturn);
   }
   else
   {
      DSL_CPE_FPrintf (out,
         "nReturn=%d DSL_DriverVersionApi=%s DSL_ChipSetFWVersion=%s "
         "DSL_ChipSetHWVersion=%s DSL_ChipSetType=%s DSL_DriverVersionMeiBsp=%s"
         DSL_CPE_CRLF, pData.accessCtl.nReturn, pData.data.DSL_DriverVersionApi,
         pData.data.DSL_ChipSetFWVersion, pData.data.DSL_ChipSetHWVersion,
         pData.data.DSL_ChipSetType, pData.data.DSL_DriverVersionMeiBsp);
   }

   return 0;
}

DSL_void_t DSL_CPE_CLI_DeviceCommandsRegister (DSL_void_t)
{
   /* Debug functionalities */
   DSL_CPE_CLI_CommandAdd ("cr", "CmvRead", DSL_CPE_CLI_DBG_CmvRead, g_sCr);
   DSL_CPE_CLI_CommandAdd ("cw", "CmvWrite", DSL_CPE_CLI_DBG_CmvWrite, g_sCw);
   DSL_CPE_CLI_CommandAdd ("dms", "DeviceMessageSend", DSL_CPE_CLI_DBG_DeviceMessageSend, g_sDms);
   DSL_CPE_CLI_CommandAdd ("vig", "VersionInformationGet", DSL_CPE_CLI_VersionInformationGet, g_sVig);
}

#endif /* INCLUDE_DSL_CPE_CLI_SUPPORT */

#endif /* #if defined (INCLUDE_DSL_CPE_API_DANUBE) || (INCLUDE_DSL_CPE_API_AMAZON_SE) ||
              defined (INCLUDE_DSL_CPE_API_AMAZON_S)*/
