/******************************************************************************
**
** FILE NAME    : dwc_otg_ifx.c
** PROJECT      : Twinpass/Danube
** MODULES      : DWC OTG USB
**
** DATE         : 12 Auguest 2007
** AUTHOR       : Sung Winder
** DESCRIPTION  : Platform specific initialization.
** COPYRIGHT    :       Copyright (c) 2007
**                      Infineon Technologies AG
**                      2F, No.2, Li-Hsin Rd., Hsinchu Science Park,
**                      Hsin-chu City, 300 Taiwan.
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Date             $Author         $Comment
** 12 Auguest 2007   Sung Winder     Initiate Version
*******************************************************************************/
#include "dwc_otg_ifx.h"

#ifdef LINUX26
#include <linux/platform_device.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#endif

#if defined(CONFIG_TWINPASS) || defined(CONFIG_DANUBE)
void dwc_otg_power_on (void)
{
    // set clock gating
    set_bit (4, (volatile unsigned long *)DANUBE_CGU_IFCCR);
    set_bit (5, (volatile unsigned long *)DANUBE_CGU_IFCCR);

    // set power
    clear_bit (0, (volatile unsigned long *)DANUBE_PMU_PWDCR);
    clear_bit (6, (volatile unsigned long *)DANUBE_PMU_PWDCR);
    clear_bit (15, (volatile unsigned long *)DANUBE_PMU_PWDCR);
#if defined (DWC_HOST_ONLY)
    // make the hardware be a host controller (default)
    clear_bit (DANUBE_USBCFG_HDSEL_BIT, (volatile unsigned long *)DANUBE_RCU_UBSCFG);
#elif defined (DWC_DEVICE_ONLY)
    /* set the controller to the device mode */
    set_bit (DANUBE_USBCFG_HDSEL_BIT, (volatile unsigned long *)DANUBE_RCU_UBSCFG);
#else
#error  "For Danube/Twinpass, it should be HOST or Device Only."
#endif

    // set the HC's byte-order to big-endian
    set_bit (DANUBE_USBCFG_HOST_END_BIT, (volatile unsigned long *)DANUBE_RCU_UBSCFG);
    clear_bit (DANUBE_USBCFG_SLV_END_BIT, (volatile unsigned long *)DANUBE_RCU_UBSCFG);

    // PHY configurations.
    writel (0x14014, (volatile unsigned long *)0xbe10103c);
}

#elif defined(CONFIG_AMAZON_SE) // defined(CONFIG_TWINPASS) || defined(CONFIG_DANUBE)

void dwc_otg_power_on (void)
{
    // set clock gating
//    clear_bit (4, (volatile unsigned long *)AMAZON_SE_CGU_IFCCR);
    clear_bit (5, (volatile unsigned long *)AMAZON_SE_CGU_IFCCR);

    // set power
    clear_bit (0, (volatile unsigned long *)AMAZON_SE_PMU_PWDCR);
    clear_bit (6, (volatile unsigned long *)AMAZON_SE_PMU_PWDCR);
    clear_bit (15, (volatile unsigned long *)AMAZON_SE_PMU_PWDCR);
#if defined (DWC_HOST_ONLY)
    // make the hardware be a host controller (default)
    clear_bit (AMAZON_SE_USBCFG_HDSEL_BIT, (volatile unsigned long *)AMAZON_SE_RCU_UBSCFG);
#elif defined (DWC_DEVICE_ONLY)
    /* set the controller to the device mode */
    set_bit (AMAZON_SE_USBCFG_HDSEL_BIT, (volatile unsigned long *)AMAZON_SE_RCU_UBSCFG);
#else
#error  "For Danube/Twinpass, it should be HOST or Device Only."
#endif

    // set the HC's byte-order to big-endian
    set_bit (AMAZON_SE_USBCFG_HOST_END_BIT, (volatile unsigned long *)AMAZON_SE_RCU_UBSCFG);
    clear_bit (AMAZON_SE_USBCFG_SLV_END_BIT, (volatile unsigned long *)AMAZON_SE_RCU_UBSCFG);

    // PHY configurations.
    writel (0x14014, (volatile unsigned long *)0xbe10103c);
}

#else // CONFIG_AMAZON_SE
#error "Please choose one platform!!"
#endif // CONFIG_AMAZON_SE

#ifdef LINUX26
static void release_platform_dev(struct device * dev)
{
	printk("IFX dwc_otg USB platform_dev release\n");
	dev->parent = NULL;
}

static struct resource resources[] =
{
	[0] = { // .name = "irq"
		// .start	= IFX_USB_IRQ,
		.flags	= IORESOURCE_IRQ,
	},
	[1] = {
		// .name   = "address",
		// .start		= IFX_USB_IOMEM_BESE,
		// .end		= IFX_USB_IOMEM_BESE+SZ_256K,
		.flags	= IORESOURCE_MEM,
	},
};

static u64 dwc_dmamask = (u32)0x1fffffff;

static struct platform_device platform_dev = {
	.id			= -1,
	.dev = {
		.release       = release_platform_dev,
		.dma_mask      = &dwc_dmamask,
	},
	.resource		= resources,
	.num_resources		= ARRAY_SIZE(resources),
};

extern const char dwc_driver_name[];
int ifx_usb_hc_init(unsigned long base_addr, int irq)
{
	if (platform_dev.dev.parent)
		return -EBUSY;

	/* finish seting up the platform device */
	resources[0].start = irq;

	resources[1].start = base_addr;
	resources[1].end = base_addr + SZ_256K;

	/* The driver core will probe for us.  We know sl811-hcd has been
	 * initialized already because of the link order dependency.
	 */
	platform_dev.name = dwc_driver_name;

	return platform_device_register(&platform_dev);
}

void ifx_usb_hc_remove(void)
{
    platform_device_unregister(&platform_dev);
}
#endif // LINUX26
